#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Script para descargar archivos desde goear.
# Copyright (C) 2007-2010 Daniel Fuentes B.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# ===================================================================
# Importamos algunos módulos que luego usaremos en el script:
# ===================================================================

import sys
import urllib
import urllib2
from xml.dom import minidom

# ===================================================================
# Algunas variables a usar:
# ===================================================================

__version__ = "0.6"
quiet_mode = False

# El texto que se muestra en la ayuda
texto_ayuda = """Script para descargar archivos de goear
Modo de uso:
goear-dl.py [Opción] [Audio_Url o ID]

Donde [Audio_Url] es la dirección (o url) del archivo, la cual es
de la siguiente forma:

http://www.goear.com/listen.php?v=xxxxxxx
o
http://www.goear.com/listen/xxxxx/cancion-artista

También se puede ingresar la ID directamente en vez de la url (la
ID es un código de 7 números/letras intercalados).

Y en [Opción] se puede ingresar una de las siguientes opciones:

Opciones disponibles:
-h, --help        Muestra la ayuda
-v, --version     Muestra la versión del programa
-q, --quiet       Omite el progreso de la descarga

Ejemplos:

goear-dl.py http://www.goear.com/listen.php?v=xxxxxxx
goear-dl.py http://www.goear.com/listen/xxxxx/cancion-artista
 # Descarga el archivo desde esa url

goear-dl.py -q http://www.goear.com/listen.php?v=xxxxx
goear-dl.py -q http://www.goear.com/listen/xxxxx/cancion-artista
 # Descarga sin mostrar el progreso

goear-dl.py xxxxxxx    # Descarga ingresando solo a ID (que es xxxxxxx)

Para mayor información revise el archivo "README"."""

# ===================================================================
# Funciones y clases usadas en el programa:
# ===================================================================


class Goear():
    """Clase que obtiene la informacion del archivo (nombre, artista, etc)"""

    def __init__(self, audio_id):
        self.id = audio_id
        self.data = None
        # Para que funcione la clase, al incio hay que obtener la informacion
        # del archivo xml a partir de la ID (primero abrir el xml)
        url_xml = "http://www.goear.com/localtrackhost.php?f=" + self.id
        # Nota: Normalmente cambian la linea anterior para que los
        # programas no funcionen al no encortrar el xml

        try:
            xmldoc = urllib2.urlopen(url_xml, None)
            xml_data = minidom.parse(xmldoc)
            xmldoc.close()
            data = xml_data.toxml()
        except: # Mensaje en caso de error
            print "Error, no se encuentra la informacion del archivo en Goear"
            print "Por favor revise la ayuda con la opción --help"
            sys.exit()

        self.data = data # Contiene la copia de informacion del xml

    def get_id(self):
        "muestra la ID de la cancion"
        return self.id

    def url_file(self):
        "devuelve la url del archivo mp3"
        # Buscar la linea "song path=" que es la dirección del archivo
        inicio_path = self.data.find("path=")
        fin_path = self.data.find(".mp3")
        path = self.data[(inicio_path + 6):(fin_path + 4)]
        return path

    def artist(self):
        "devuelve el grupo/artista de esta cancion"
        # Buscar la linea "title=" que es la dirección del archivo
        inicio = self.data.find("artist=")
        fin = self.data.find("\"", (inicio + 8), -1)
        artist = self.data[(inicio + 8):fin]
        return artist

    def title(self):
        "devuelve el titulo de la cancion"
        # Buscar la linea "title=" que es la dirección del archivo
        inicio = self.data.find("title=")
        fin = self.data.find("\"", (inicio + 7), -1)
        title = self.data[(inicio + 7):fin]
        return title

# -------------------------------------------------------------------
# Definimos la función de descarga:


def download_file(filename, URL, quiet=False):
    if quiet == True:
        f = urllib.urlretrieve(URL, filename)
    else:
        f = urllib.urlretrieve(URL, filename, reporthook=my_report_hook)


def my_report_hook(block_count, block_size, total_size):
    """función que muestra el progreso del archivo"""
    # Usada si al descargar se pide mostrar el progreso (por defecto)
    total_kb = total_size / 1024
    descargado = block_count * (block_size / 1024)
    porcentaje = descargado * 100 / total_kb
    # Mostramos en el teminal la salida, se usa stdout porque no
    # inserte el alto de linea "\n" a diferencia de print
    texto_barra = "Descargados " + str(descargado) + " kb de " +\
    str(total_kb) + " kb (" + str(porcentaje) + "%)"
    # Borramos la linea anterior (asi se actualiza la misma linea)
    sys.stdout.write("\b" * (len(texto_barra)))
    sys.stdout.write(texto_barra)
    sys.stdout.flush()


# -------------------------------------------------------------------
# Funcion que obtiene la ID a partir de la dirección ingresada


def get_id(url):
    # hay varios posibles casos:
    # a) Se ingresa directamente la ID (codigo de 7 numeros/letras)
    # b) De la forma antigua (http://www.goear.com/listen.php?v=xxxxx):
    # c) De la forma nueva (http://www.goear.com/listen/xxxxx/cancion-artista)

    if len(audio_url) == 7:
        # se ingreso la id directamente (primer caso)
        audio_id = audio_url
        return audio_id

    elif (audio_url.find("listen.php?v=")) != -1: # Segundo caso
        # de la forma antigua el find deberia encontrar el "listen.php?v="
        # siendo distintinto de -1 (es -1 cuando no encuentra el texto pedido)
        inicio_id = audio_url.find("v=")
        audio_id = audio_url[(inicio_id + 2):(inicio_id + 9)]
        return audio_id

    elif (audio_url.find("listen/")) != -1:
        # la nueva forma (tercer caso)
        inicio_id = audio_url.find("listen/")
        audio_id = audio_url[(inicio_id + 7):(inicio_id + 14)]
        return audio_id

    else:
        print "Error: no se encuentra la direccion o ID de:\n%s" % (audio_url)
        sys.exit()


# -------------------------------------------------------------------
# funcion que comprueba las opciones ingresadas, actua de acuerdo a ella y
# regresa la url de goear (si es necesaria) o termina el programa


def checkoptions():
    """Comprueba las opciones, las ejecuta y regresa el audio_url"""
    # Variables a usar en la funcion (una ya esta definida, la global)
    global quiet_mode
    opciones_ingresadas = []

#...................................................................
# Primero se comprueba la entrada de datos, hay dos posibles casos:
    # 1) Se ha ingresa solo 1 opcion (o la url)
    # 2) Se ingresan muchas opciones

    # Caso 1:
    if len(sys.argv) == 2:
    # Para el caso 1, hay dos valores posibles:
    # se ingreso una opcion o se ingreso solo la url sin ninguna opcion
        if (sys.argv[1] == "-q" or sys.argv[1] == "--quiet" or
 sys.argv[1] == "-h" or sys.argv[1] == "--help" or sys.argv[1] == "help" or
 sys.argv[1] == "-v" or sys.argv[1] == "--version" or sys.argv[1] == "moo"):
            opciones_ingresadas.append(sys.argv[1])
            audio_url = None
        else:
            audio_url = sys.argv[1]

    # Caso 2: Se ingresan muchas opciones
    elif len(sys.argv) > 2:
    # se pueden tener solo ordenes o tener opciones (ordenes) + dirección
        for n in sys.argv[1:len(sys.argv)]:
            if (n == "-q" or n == "--quiet" or n == "-h" or n == "--help"
 or n == "help" or n == "-v" or n == "--version" or n == "moo"):
                opciones_ingresadas.append(n)
                audio_url = None
            else:
                audio_url = n

#...................................................................
# Luego hay que verificar las Opciones de la linea de comandos
# y proceder de acuerdo a ellas
    if opciones_ingresadas != []:
        for opcion in opciones_ingresadas:
            if opcion == "-h" or opcion == "--help" or opcion == "help":
                print texto_ayuda
                sys.exit()
            elif opcion == "-v" or opcion == "--version":
                print "Ver:", __version__
                sys.exit()
            elif opcion == "-q" or opcion == "--quiet":
                quiet_mode = True
            elif opcion == "moo":
                import time
                print "No hay ninguna Súper Vaca en este programa, pero"
                print "tenemos una danza:"
                sys.stdout.write(" <(^.^<) ")
                sys.stdout.flush()
                time.sleep(2)
                sys.stdout.write("\b\b\b\b\b\b\b\b\b")
                sys.stdout.write(" <(^-^)> ")
                sys.stdout.flush()
                time.sleep(2)
                sys.stdout.write("\b\b\b\b\b\b\b\b\b")
                sys.stdout.write(" (>^.^)> ")
                sys.stdout.flush()
                time.sleep(2)
                sys.stdout.write("\b\b\b\b\b\b\b\b\b")
                sys.stdout.write(" ^(^.^)^\n")
                sys.stdout.flush()
                sys.exit()

#...................................................................
# Por ultimo comprobar la dirección (url) del audio sea correcta:
    # Puede darse casos como el que sigue: goear.py -q -q (bien raro)
    if audio_url == None:
        audio_url = str(raw_input("Ingrese la dirección (url) de \
goear o su ID: "))

    return audio_url


#====================================================================
# Iniciamos el programa
#====================================================================

if __name__ == "__main__":
    # Comprobar a continuación si se ingreso algun dato en la linea de ordenes
    # Si no se ingresa nada sys.argv solo tendra el nombre del script
    if len(sys.argv) == 1:
        # No hay ordenes o url, por lo que hay que pedir la dirección
        audio_url = str(raw_input("Ingrese la dirección (url) del \
archivo o su ID: "))
    else:
        audio_url = checkoptions()

    # Se obtiene la ID de la cancion
    audio_id = get_id(audio_url)
    print "La ID de esta canción es: %s" % (audio_id)

    # Luego de esto se procede a las descarga
    cancion = Goear(audio_id)
    # Guardamos el nombre del archivo
    nombre_archivo = (cancion.get_id() + " - " + cancion.title() + " - " +
cancion.artist() + ".mp3")
    print "Iniciando descarga de: %s (%s)" % (audio_id, cancion.title())
    download_file(nombre_archivo, cancion.url_file(), quiet_mode)
    print "\nListo, ahora solo abra el archivo \
llamado:\n\"%s\"" % (nombre_archivo)
